

;-----------------------------------
; declarations
;-----------------------------------

extensions [nlboris]

globals
[ portal                      ; the boris portal
  cleaner-carrying-color      ; as it says
  cleaner-normal-color        ; ditto
  bin-x                       ; xcor of bin
  bin-y                       ; ycor of bin
]



;-----------------------------------
; breed data
;-----------------------------------


breed [bins bin]

breed [cleaners clearner]     ;____ cleaner _______
cleaners-own
[ status          ; waiting/fetching/carrying
  target-x        ; xcor of mine they are after
  target-y        ; ycor of mine they are after
  carry-mine      ; the mine they are carrying
  name
]


breed [mines mine]            ;____ mine __________
mines-own
[ status          ; hidden/found/to-clean/cleaned
]


breed [detectors detector]    ;____ detector _______
detectors-own
[ status          ; tracking/(looking)
  target          ; which mine they are tracking
]


;-----------------------------------
; general
;-----------------------------------

to setup
  clear-all
  setup-globals
  setup-patches
  setup-mines
  setup-bins
  setup-detectors
  setup-cleaners
end

to go
  if not any? mines
  [ print (list "ticks used:" ticks)
    stop
  ]
  move-detectors
  move-cleaners
  tick
end


;-----------------------------------
; setup for breeds, etc
;-----------------------------------

to setup-patches
  ask patches [set pcolor gray]
end

to setup-globals
  ; set globals
  set portal nlboris:make "P1"
  set cleaner-carrying-color red
  set cleaner-normal-color yellow
end

to setup-mines
  create-mines no-of-mines
  ask mines
  [
    setxy random-xcor random-ycor
    set color black
    set shape "ufo top"
    set status "hidden"
  ]
end

to setup-bins
  create-bins 1    ;; one bin only
  ask bins
  [
    setxy bin-x bin-y
    set color black
    set shape "garbage can"
  ]
end

to setup-detectors
  create-detectors no-of-detectors
  ask detectors
  [ nlboris:register portal (word "detect" who)
    setxy 0 0
    set color blue
    set status "looking"
  ]
end

to setup-cleaners
  create-cleaners no-of-cleaners
  ask cleaners
  [ set name (word "clean" who)
    nlboris:register portal name
    setxy 0 0
    set color cleaner-normal-color
    set status "waiting"
  ]
end


;-----------------------------------
; movement for breeds
;-----------------------------------

to move-detectors
  ;; detectors move around reporting position of mines
  ;; they are either tracking a found mine or looking for their next one
  ask detectors
  [ ifelse status = "tracking"       ;________ TRACKING ________________
    [ ifelse distance target < 1
      [ ; found a mine - tell one of the cleaners where it is & mark it "to-clean"
        let cleaner [name] of (one-of cleaners)
        nlboris:send-msg portal cleaner (list ([xcor] of target) ([ycor] of target))
        ask target [set status "to-clean"]
        set status "looking"
      ]
      [ ; keep tracking
        if target != nobody
        [ face target
          fd 1
        ]
      ]
    ]
    [ ;else status = "looking"        __________ LOOKING ________________
      set target one-of mines with [status = "hidden"]
      if target != nobody
      [ set status "tracking"               ;; mark this detector as "tracking" now
        ask target [set status "found"]     ;; mark mine as "found"
      ]
    ]
  ]
end


to move-cleaners
  ;; cleaners get told about mines by detectors then go get them
  ;; & carry them back to the bin
  ;; cleaners can be waiting/fetching/carrying
  ask cleaners
  [ ifelse status = "waiting"           ;________ WAITING ________________
    [ if nlboris:is-msg-waiting portal
      [ set status "fetching"
        
        let m nlboris:get-msg portal
        
        let from item 0 m
        let msg  item 1 m
        set target-x   item 0 msg
        set target-y   item 1 msg
      ]
    ]
    [ ifelse status = "fetching"         ;________ FETCHING ________________
                                         ; ie: progressing to a detected mine
      [ ifelse distancexy target-x target-y < 1
        [ ; pick it up & start taking it to the bin
          set carry-mine one-of mines with
              [(xcor = [target-x] of myself) and (ycor = [target-y] of myself)
                and (status = "to-clean")]
          ask carry-mine [hide-turtle]
          set color cleaner-carrying-color
          set status "carrying"
        ]
        [ ;; carry on tracking
          facexy target-x target-y
          fd 1
        ]
      ]
      [ if status = "carrying"           ;________ CARRYING ________________
        [ facexy bin-x bin-y             ; ie: carrying mine to the bin
          fd 1
          if any? bins-here
          [ ask carry-mine [die]
            set status "waiting"
            set color cleaner-normal-color
          ]
        ]
      ]
    ]
  ]
end





  

@#$#@#$#@
GRAPHICS-WINDOW
187
17
730
581
20
20
13.0
1
10
1
1
1
0
1
1
1
-20
20
-20
20
1
1
1
ticks

BUTTON
17
20
80
53
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
96
68
159
101
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL

BUTTON
18
67
79
100
go once
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL

SLIDER
15
112
158
145
no-of-mines
no-of-mines
1
50
30
1
1
NIL
HORIZONTAL

SLIDER
14
155
159
188
no-of-detectors
no-of-detectors
1
16
4
1
1
NIL
HORIZONTAL

SLIDER
15
200
159
233
no-of-cleaners
no-of-cleaners
1
10
3
1
1
NIL
HORIZONTAL

@#$#@#$#@
WHAT IS IT?
-----------
The scenario is this... the environment contains multiple mines and one bin. Mines can be made harmless by putting them in the bin (dont try this at home). When all the mines are in the bin the model stops running.


HOW IT WORKS
------------
There are two types of agent: detectors and cleaners. Detectors have sensors which can give the direction of a mine but they can only determine a mines exact position by moving up to it. Detectors are not capable of picking up mines so cannot carry them to the bin.

Cleaners have no sensing capabilities so cannot locate mines on their own but they are able to pick up mines (one at a time) & carry them to the bin.

Detectors and cleaners work together: detectors find mines then send messages to cleaners to tell the cleaners where the mines are located. Cleaners get messages from detectors containing information about the location of mines then go pick up those mines and carry them to the bin.

Note: this is roughly similar to an example used in a training session I once attended for building BDI agents in 3APL so the credit for the nature of this example goes to the authors of the 3APL training materials. Our model is quite different to the kind of solution you would build in 3APL (we are not using BDI agents) but the example is nice to use for investigating agent messaging.


HOW TO USE IT
-------------
The model allows users to specify the number of mines, detectors & cleaners. The bin is in the middle of the environment (xcor=0, ycor=0) and all of the agents start there. Mines are randomly scattered around the environment.

Mines are circular black & white blobs, the bin is  a green garbage-can shape, detectors are blue (standard turtle shape), cleaners are yellow when not carrying a mine and red when they are carrying.

When the model starts running, detectors (blue) leave the bin area, each heading for a mine. When a detector reaches a mine it selects a cleaner (at random) and sends it a message about the mines location. The cleaner (yellow) then leaves the bin area to fetch the mine and the detector moves off towards another mine.

When the cleaner reaches a mine the cleaner turns red and the mine symbol disappears from the visual environment, the cleaner then takes the mine to the bin. When a (red) cleaner holding a mine reaches the bin the cleaner turns yellow & checks if it has a message waiting from a detector about another mine that needs putting in the bin.


THINGS TO NOTICE
----------------
Notice (i) the detectors inevitably finish their work before the cleaners (ii) some cleaners do more work than others because the detectors randomly choose which cleaners to inform about mines (so some finish before others).


FURTHER INFORMATION
--------------
See the NL Boris tutorial


CREDITS AND REFERENCES
----------------------
this model was developed as part of the tutorial for NL Boris - see www.agent-domain.org foradditional tutorials and updates
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

garbage can
false
0
Polygon -16777216 false false 60 240 66 257 90 285 134 299 164 299 209 284 234 259 240 240
Rectangle -7500403 true true 60 75 240 240
Polygon -7500403 true true 60 238 66 256 90 283 135 298 165 298 210 283 235 256 240 238
Polygon -7500403 true true 60 75 66 57 90 30 135 15 165 15 210 30 235 57 240 75
Polygon -7500403 true true 60 75 66 93 90 120 135 135 165 135 210 120 235 93 240 75
Polygon -16777216 false false 59 75 66 57 89 30 134 15 164 15 209 30 234 56 239 75 235 91 209 120 164 135 134 135 89 120 64 90
Line -16777216 false 210 120 210 285
Line -16777216 false 90 120 90 285
Line -16777216 false 125 131 125 296
Line -16777216 false 65 93 65 258
Line -16777216 false 175 131 175 296
Line -16777216 false 235 93 235 258
Polygon -16777216 false false 112 52 112 66 127 51 162 64 170 87 185 85 192 71 180 54 155 39 127 36

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

tree pine
false
0
Rectangle -6459832 true false 120 225 180 300
Polygon -7500403 true true 150 240 240 270 150 135 60 270
Polygon -7500403 true true 150 75 75 210 150 195 225 210
Polygon -7500403 true true 150 7 90 157 150 142 210 157 150 7

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

ufo top
false
0
Circle -1 true false 15 15 270
Circle -16777216 false false 15 15 270
Circle -7500403 true true 75 75 150
Circle -16777216 false false 75 75 150
Circle -7500403 true true 60 60 30
Circle -7500403 true true 135 30 30
Circle -7500403 true true 210 60 30
Circle -7500403 true true 240 135 30
Circle -7500403 true true 210 210 30
Circle -7500403 true true 135 240 30
Circle -7500403 true true 60 210 30
Circle -7500403 true true 30 135 30
Circle -16777216 false false 30 135 30
Circle -16777216 false false 60 210 30
Circle -16777216 false false 135 240 30
Circle -16777216 false false 210 210 30
Circle -16777216 false false 240 135 30
Circle -16777216 false false 210 60 30
Circle -16777216 false false 135 30 30
Circle -16777216 false false 60 60 30

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 4.1.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment" repetitions="20" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <enumeratedValueSet variable="smart-foxes-on">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-of-hounds">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-of-foxes">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="smart-hounds-on">
      <value value="true"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="hound-memory" repetitions="1" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <metric>count turtles</metric>
    <enumeratedValueSet variable="smart-foxes-on">
      <value value="true"/>
    </enumeratedValueSet>
    <steppedValueSet variable="global-memory-duration" first="15" step="1" last="25"/>
    <enumeratedValueSet variable="no-of-hounds">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-of-foxes">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="smart-hounds-on">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="total-ticks">
      <value value="0"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiment" repetitions="10" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <enumeratedValueSet variable="smart-foxes-on">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-of-hounds">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="no-of-foxes">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="smart-hounds-on">
      <value value="true"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
